package de.fraunhofer.iem.dataprovider.taskManager.tasks

import de.fraunhofer.iem.dataprovider.gitlab.GitConfiguration
import de.fraunhofer.iem.dataprovider.taskManager.Event
import de.fraunhofer.iem.dataprovider.taskManager.SequenceTaskDoneEvent
import org.gitlab4j.api.GitLabApi
import java.util.*

sealed class GetGitlabProjectTask(
    override val type: TaskType = TaskType.REPO_CHANGED,
    protected open val repoId: Any,
    private val gitlabConfiguration: GitConfiguration,
    private val responseChannel: suspend (task: Event) -> Unit,
    override val sequenceId: UUID? = null
): Task() {

    private val gitlabApi: GitLabApi = GitLabApi(gitlabConfiguration.host, gitlabConfiguration.accessToken)
    private lateinit var gitProject: GitProject
    override suspend fun execute() {
        logger.info("Gitlab config ${gitlabConfiguration.host} ${gitlabConfiguration.accessToken}")
        val project = gitlabApi.projectApi.getProject(repoId)
        logger.info(project.toString())
        val projectUri = project.httpUrlToRepo
        gitProject = GitProject(project.name, projectUri)
        logger.info("Retrieved project ${project.path} and url $projectUri")
        if (sequenceId != null) {
            responseChannel(SequenceTaskDoneEvent(sequenceId!!))
        }
//        responseChannel(GitTask(this.type, gitProject, responseChannel))
    }

    fun getResult(): GitProject {
        return gitProject
    }
}

class GetGitlabProjectPathTask(
    override val repoId: String,
    private val gitlabConfiguration: GitConfiguration,
    private val responseChannel: suspend (task: Event) -> Unit,
    override val sequenceId: UUID? = null
): GetGitlabProjectTask(repoId = repoId, gitlabConfiguration = gitlabConfiguration, responseChannel = responseChannel )

class GetGitlabProjectIdTask(
    override val repoId: Long,
    private val gitlabConfiguration: GitConfiguration,
    private val responseChannel: suspend (task: Event) -> Unit,
    override val sequenceId: UUID? = null
): GetGitlabProjectTask(repoId = repoId, gitlabConfiguration = gitlabConfiguration, responseChannel = responseChannel )
