package de.fraunhofer.iem.dataprovider.taskManager.tasks

import de.fraunhofer.iem.dataprovider.taskManager.Event
import de.fraunhofer.iem.dataprovider.taskManager.SequenceTaskDoneEvent
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.util.*

sealed class ProcessTask(protected open vararg val flags: String): Task() {

    protected abstract val execPath: String
    private val mainScope = CoroutineScope(Dispatchers.Default)
    override suspend fun execute() {

        val p = ProcessBuilder(execPath, *flags)
            .start()

        p.onExit().thenApply { p1 ->
            mainScope.launch {
                handleProcessReturn(p1)
            }
        }
    }

    abstract suspend fun handleProcessReturn(p: Process)
}

/**
 * Dummy Task to simulate running external processes.
 */
class JavaTask(
    override val type: TaskType,
    override vararg val flags: String,
    private val responseChannel: suspend (task: Event) -> Unit,
    override val sequenceId: UUID?
): ProcessTask() {

    override val execPath: String = "java"
    override suspend fun handleProcessReturn(p: Process) {
//        responseChannel(DoneTask("Done with external process. Exit value ${p.exitValue()}"))
        if (sequenceId != null) {
            responseChannel(SequenceTaskDoneEvent(sequenceId!!))
        }
    }
}